//go:build windows
// +build windows

package service

import (
	"archive/zip"
	"errors"
	"fyne.io/fyne/v2"
	"fyne.io/fyne/v2/canvas"
	"fyne.io/fyne/v2/lang"
	"fyne.io/fyne/v2/widget"
	"git.kor-elf.net/kor-elf/gui-for-ffmpeg/internal/application"
	"io"
	"net/http"
	"os"
	"path/filepath"
	"strings"
)

func startDownload(app application.AppContract, progressBar *widget.ProgressBar, progressMessage *canvas.Text, save func(ffmpegPath string, ffprobePath string, ffplayPath string) error) error {
	var err error

	dir := os.Getenv("APPDATA")
	dir = filepath.Join(dir, "fyne", app.FyneApp().UniqueID())
	err = os.MkdirAll(dir, 0755)
	if err != nil {
		return err
	}

	fyne.Do(func() {
		progressMessage.Text = lang.L("downloadRun")
		progressMessage.Refresh()
	})
	err = downloadFile(dir+"/ffmpeg.zip", "https://github.com/BtbN/FFmpeg-Builds/releases/download/latest/ffmpeg-master-latest-win64-gpl.zip", progressBar)
	if err != nil {
		return err
	}

	fyne.Do(func() {
		progressMessage.Text = lang.L("unzipRun")
		progressMessage.Refresh()
	})
	err = unZip(dir+"/ffmpeg.zip", dir, progressBar)
	if err != nil {
		return err
	}
	_ = os.Remove(dir + "/ffmpeg.zip")

	fyne.Do(func() {
		progressMessage.Text = lang.L("testFF")
		progressMessage.Refresh()
	})
	err = save(
		dir+"/ffmpeg-master-latest-win64-gpl/bin/ffmpeg.exe",
		dir+"/ffmpeg-master-latest-win64-gpl/bin/ffprobe.exe",
		dir+"/ffmpeg-master-latest-win64-gpl/bin/ffplay.exe",
	)
	if err != nil {
		return err
	}

	fyne.Do(func() {
		progressMessage.Text = lang.L("completedQueue")
		progressMessage.Refresh()
	})

	return nil
}

func downloadFile(filepath string, url string, progressBar *widget.ProgressBar) (err error) {
	progressBar.Value = 0
	progressBar.Max = 100

	req, err := http.NewRequest("GET", url, nil)
	if err != nil {
		return err
	}
	resp, err := http.DefaultClient.Do(req)
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	f, err := os.OpenFile(filepath, os.O_CREATE|os.O_WRONLY, 0644)
	if err != nil {
		return err
	}
	defer f.Close()

	buf := make([]byte, 32*1024)
	var downloaded int64
	for {
		n, err := resp.Body.Read(buf)
		if err != nil {
			if err == io.EOF {
				break
			}
			return err
		}
		if n > 0 {
			f.Write(buf[:n])
			downloaded += int64(n)
			progressBar.Value = float64(downloaded) / float64(resp.ContentLength) * 100
			fyne.Do(func() {
				progressBar.Refresh()
			})
		}
	}
	return nil
}

func unZip(fileZip string, directory string, progressBar *widget.ProgressBar) error {
	progressBar.Value = 0
	progressBar.Max = 100

	fyne.Do(func() {
		progressBar.Refresh()
	})

	archive, err := zip.OpenReader(fileZip)
	if err != nil {
		return err
	}
	defer archive.Close()

	totalBytes := int64(0)
	for _, f := range archive.File {
		totalBytes += int64(f.UncompressedSize64)
	}

	unpackedBytes := int64(0)

	for _, f := range archive.File {
		filePath := filepath.Join(directory, f.Name)

		if !strings.HasPrefix(filePath, filepath.Clean(directory)+string(os.PathSeparator)) {
			return errors.New("invalid file path")
		}
		if f.FileInfo().IsDir() {
			os.MkdirAll(filePath, os.ModePerm)
			continue
		}

		if err := os.MkdirAll(filepath.Dir(filePath), os.ModePerm); err != nil {
			return err
		}

		dstFile, err := os.OpenFile(filePath, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, f.Mode())
		if err != nil {
			return err
		}

		fileInArchive, err := f.Open()
		if err != nil {
			return err
		}

		bytesRead, err := io.Copy(dstFile, fileInArchive)
		if err != nil {
			return err
		}

		unpackedBytes += bytesRead
		progressBar.Value = float64(unpackedBytes) / float64(totalBytes) * 100
		fyne.Do(func() {
			progressBar.Refresh()
		})

		dstFile.Close()
		fileInArchive.Close()
	}

	return nil
}
